@@ -785,4 +785,30 @@ AddonManager::check_for_langpack_updates()
   }
 }
 
+#ifdef EMSCRIPTEN
+void
+AddonManager::onDownloadProgress(int id, int loaded, int total)
+{
+  m_downloader.onDownloadProgress(id, loaded, total);
+}
+
+void
+AddonManager::onDownloadFinished(int id)
+{
+  m_downloader.onDownloadFinished(id);
+}
+
+void
+AddonManager::onDownloadError(int id)
+{
+  m_downloader.onDownloadError(id);
+}
+
+void
+AddonManager::onDownloadAborted(int id)
+{
+  m_downloader.onDownloadAborted(id);
+}
+#endif
+
 /* EOF */
