@@ -25,7 +25,7 @@ class WavSoundFile final : public SoundFile
 {
 public:
   WavSoundFile(PHYSFS_file* file);
-  ~WavSoundFile();
+  ~WavSoundFile() override;
 
   virtual size_t read(void* buffer, size_t buffer_size) override;
   virtual void reset() override;
