@@ -64,13 +64,13 @@ BadGuy::BadGuy(const Vector& pos, Direction direction, const std::string& sprite
   m_melting_time(0),
   m_lightsprite(SpriteManager::current()->create(light_sprite_name)),
   m_glowing(false),
+  m_parent_dispenser(),
   m_state(STATE_INIT),
   m_is_active_flag(),
   m_state_timer(),
   m_on_ground_flag(false),
-  m_floor_normal(),
-  m_colgroup_active(COLGROUP_MOVING),
-  m_parent_dispenser()
+  m_floor_normal(0.0f, 0.0f),
+  m_colgroup_active(COLGROUP_MOVING)
 {
   SoundManager::current()->preload("sounds/squish.wav");
   SoundManager::current()->preload("sounds/fall.wav");
@@ -98,13 +98,13 @@ BadGuy::BadGuy(const ReaderMapping& reader, const std::string& sprite_name_, int
   m_melting_time(0),
   m_lightsprite(SpriteManager::current()->create(light_sprite_name)),
   m_glowing(false),
+  m_parent_dispenser(),
   m_state(STATE_INIT),
   m_is_active_flag(),
   m_state_timer(),
   m_on_ground_flag(false),
-  m_floor_normal(),
-  m_colgroup_active(COLGROUP_MOVING),
-  m_parent_dispenser()
+  m_floor_normal(0.0f, 0.0f),
+  m_colgroup_active(COLGROUP_MOVING)
 {
   std::string dir_str = "auto";
   reader.get("direction", dir_str);
@@ -168,7 +168,7 @@ BadGuy::update(float dt_sec)
         badguy = badguy.substr(path_chars + 1, badguy.length() - path_chars);
         // log warning since badguys_killed can no longer reach total_badguys
         std::string current_level = "[" + Sector::get().get_level()->filename + "] ";
-        log_warning << current_level << "Counted badguy " << badguy << " starting at " << start_position << " has left the sector" <<std::endl;;
+        log_warning << current_level << "Counted badguy " << badguy << " starting at " << start_position << " has left the sector" <<std::endl;
       }*/
       return;
     }
@@ -196,7 +196,7 @@ BadGuy::update(float dt_sec)
 
     case STATE_BURNING: {
       m_is_active_flag = false;
-      m_col.m_movement = m_physic.get_movement(dt_sec);
+      m_col.set_movement(m_physic.get_movement(dt_sec));
       if ( m_sprite->animation_done() ) {
         remove_me();
       }
@@ -209,12 +209,12 @@ BadGuy::update(float dt_sec)
         remove_me();
         break;
       }
-      m_col.m_movement = m_physic.get_movement(dt_sec);
+      m_col.set_movement(m_physic.get_movement(dt_sec));
       break;
 
     case STATE_MELTING: {
       m_is_active_flag = false;
-      m_col.m_movement = m_physic.get_movement(dt_sec);
+      m_col.set_movement(m_physic.get_movement(dt_sec));
       if ( m_sprite->animation_done() || on_ground() ) {
         Sector::get().add<WaterDrop>(m_col.m_bbox.p1(), get_water_sprite(), m_physic.get_velocity());
         remove_me();
@@ -224,7 +224,7 @@ BadGuy::update(float dt_sec)
 
     case STATE_GROUND_MELTING:
       m_is_active_flag = false;
-      m_col.m_movement = m_physic.get_movement(dt_sec);
+      m_col.set_movement(m_physic.get_movement(dt_sec));
       if ( m_sprite->animation_done() ) {
         remove_me();
       }
@@ -232,7 +232,7 @@ BadGuy::update(float dt_sec)
 
     case STATE_INSIDE_MELTING: {
       m_is_active_flag = false;
-      m_col.m_movement = m_physic.get_movement(dt_sec);
+      m_col.set_movement(m_physic.get_movement(dt_sec));
       if ( on_ground() && m_sprite->animation_done() ) {
         m_sprite->set_action(m_dir == Direction::LEFT ? "gear-left" : "gear-right", 1);
         set_state(STATE_GEAR);
@@ -249,7 +249,7 @@ BadGuy::update(float dt_sec)
 
     case STATE_FALLING:
       m_is_active_flag = false;
-      m_col.m_movement = m_physic.get_movement(dt_sec);
+      m_col.set_movement(m_physic.get_movement(dt_sec));
       break;
   }
 
@@ -267,7 +267,7 @@ BadGuy::str2dir(const std::string& dir_str) const
     return Direction::RIGHT;
 
   //default to "auto"
-  log_warning << "Badguy::str2dir: unknown direction \"" << dir_str << "\"" << std::endl;;
+  log_warning << "Badguy::str2dir: unknown direction \"" << dir_str << "\"" << std::endl;
   return Direction::AUTO;
 }
 
@@ -289,7 +289,7 @@ BadGuy::deactivate()
 void
 BadGuy::active_update(float dt_sec)
 {
-  m_col.m_movement = m_physic.get_movement(dt_sec);
+  m_col.set_movement(m_physic.get_movement(dt_sec));
   if (m_frozen)
     m_sprite->stop_animation();
 }
@@ -315,15 +315,23 @@ BadGuy::collision_tile(uint32_t tile_attributes)
     m_in_water = false;
   }
 
-  if (tile_attributes & Tile::HURTS && is_hurtable()) {
-    if (tile_attributes & Tile::FIRE) {
-      if (is_flammable()) ignite();
-    }
-    else if (tile_attributes & Tile::ICE) {
-      if (is_freezable()) freeze();
-    }
-    else {
-      kill_fall();
+  if (tile_attributes & Tile::HURTS && is_hurtable())
+  {
+    Rectf hurtbox = get_bbox().grown(-6.f);
+    if (!Sector::get().is_free_of_tiles(hurtbox, true, Tile::HURTS) || tile_attributes & Tile::UNISOLID)
+    {
+      if (tile_attributes & Tile::FIRE)
+      {
+        if (is_flammable()) ignite();
+      }
+      else if (tile_attributes & Tile::ICE)
+      {
+        if (is_freezable()) freeze();
+      }
+      else
+      {
+        kill_fall();
+      }
     }
   }
 }
@@ -386,6 +394,15 @@ BadGuy::collision_solid(const CollisionHit& hit)
   update_on_ground_flag(hit);
 }
 
+void
+BadGuy::on_flip(float height)
+{
+  MovingObject::on_flip(height);
+  Vector pos = get_start_position();
+  pos.y = height - pos.y;
+  set_start_position(pos);
+}
+
 HitResponse
 BadGuy::collision_player(Player& player, const CollisionHit& )
 {
@@ -510,7 +527,7 @@ BadGuy::kill_fall()
 
   if (m_frozen) {
     SoundManager::current()->play("sounds/brick.wav");
-    Vector pr_pos;
+    Vector pr_pos(0.0f, 0.0f);
     float cx = m_col.m_bbox.get_width() / 2;
     float cy = m_col.m_bbox.get_height() / 2;
     for (pr_pos.x = 0; pr_pos.x < m_col.m_bbox.get_width(); pr_pos.x += 16) {
@@ -546,7 +563,7 @@ void
 BadGuy::run_dead_script()
 {
   if (m_countMe)
-    Sector::get().get_level().m_stats.m_badguys++;
+    Sector::get().get_level().m_stats.increment_badguys();
 
   m_countMe = false;
 
@@ -601,8 +618,8 @@ BadGuy::set_state(State state_)
 bool
 BadGuy::is_offscreen() const
 {
-  Vector cam_dist;
-  Vector player_dist;
+  Vector cam_dist(0.0f, 0.0f);
+  Vector player_dist(0.0f, 0.0f);
   Camera& cam = Sector::get().get_camera();
   cam_dist = cam.get_center() - m_col.m_bbox.get_middle();
   if (Editor::is_active()) {
@@ -848,6 +865,8 @@ BadGuy::get_settings()
 void
 BadGuy::after_editor_set()
 {
+  MovingSprite::after_editor_set();
+
   if (m_dir == Direction::AUTO)
   {
     if (m_sprite->has_action("editor-left")) {
