@@ -28,8 +28,6 @@ public:
   virtual bool is_freezable() const override;
   virtual std::string get_class() const override { return "ghostflame"; }
   virtual std::string get_display_name() const override { return _("Ghost Flame"); }
-  virtual ObjectSettings get_settings() override;
-  virtual void after_editor_set() override;
 
 private:
   Ghostflame(const Ghostflame&) = delete;
