@@ -28,10 +28,11 @@
 namespace {
 
 const float TIME_EXPLOSION = 5.0f;
+const float STOMPED_TIME = 1.0f;
 const float TIME_STUNNED = 0.5f;
 
 const float NORMAL_WALK_SPEED = 80.0f;
-const float EXPLODING_WALK_SPEED = 160.0f;
+const float EXPLODING_WALK_SPEED = 200.0f;
 
 } // namespace
 
@@ -42,7 +43,8 @@ Haywire::Haywire(const ReaderMapping& reader) :
   is_stunned(false),
   time_stunned(0.0f),
   ticking(),
-  grunting()
+  grunting(),
+  stomped_timer()
 {
   walk_speed = NORMAL_WALK_SPEED;
   max_drop_height = 16;
@@ -84,6 +86,7 @@ Haywire::collision_squished(GameObject& object)
 
   if (!is_exploding) {
     start_exploding();
+	stomped_timer.start(STOMPED_TIME);
   }
 
   time_stunned = TIME_STUNNED;
@@ -121,23 +124,72 @@ Haywire::active_update(float dt_sec)
     }
   }
 
-  if (is_exploding) {
+  if (is_exploding)
+  {
+    if (on_ground() && std::abs(m_physic.get_velocity_x()) > 40.f && !Sector::get().get_player().is_dying())
+    {
+      //jump over 1-tall roadblocks
+      Rectf jump_box = get_bbox();
+      jump_box.set_left(m_col.m_bbox.get_left() + (m_dir == Direction::LEFT ? -48.f : 38.f));
+      jump_box.set_right(m_col.m_bbox.get_right() + (m_dir == Direction::RIGHT ? 48.f : -38.f));
+
+      Rectf exception_box = get_bbox();
+      exception_box.set_left(m_col.m_bbox.get_left() + (m_dir == Direction::LEFT ? -48.f : 38.f));
+      exception_box.set_right(m_col.m_bbox.get_right() + (m_dir == Direction::RIGHT ? 48.f : -38.f));
+      exception_box.set_top(m_col.m_bbox.get_top() - 32.f);
+      exception_box.set_bottom(m_col.m_bbox.get_bottom() - 48.f);
+
+      if (!Sector::get().is_free_of_statics(jump_box) && Sector::get().is_free_of_statics(exception_box))
+      {
+        m_physic.set_velocity_y(-325.f);
+      }
+      else
+      {
+        //jump over gaps if Tux isnt below
+        Rectf gap_box = get_bbox();
+        gap_box.set_left(m_col.m_bbox.get_left() + (m_dir == Direction::LEFT ? -38.f : 26.f));
+        gap_box.set_right(m_col.m_bbox.get_right() + (m_dir == Direction::LEFT ? -26.f : 38.f));
+        gap_box.set_top(m_col.m_bbox.get_top());
+        gap_box.set_bottom(m_col.m_bbox.get_bottom() + 28.f);
+
+        if (Sector::get().is_free_of_statics(gap_box)
+          && (get_nearest_player()->get_bbox().get_bottom() <= m_col.m_bbox.get_bottom()))
+        {
+          m_physic.set_velocity_y(-325.f);
+        }
+      }
+    }
+
+    //end of pathfinding
+
+	  if (stomped_timer.get_timeleft() < 0.05f) {
+        set_action ((m_dir == Direction::LEFT) ? "ticking-left" : "ticking-right", /* loops = */ -1);
+        walk_left_action = "ticking-left";
+        walk_right_action = "ticking-right";
+    }
+    else {
+        set_action ((m_dir == Direction::LEFT) ? "active-left" : "active-right", /* loops = */ 1);
+        walk_left_action = "active-left";
+	      walk_right_action = "active-right";
+    }
+
     auto p = get_nearest_player ();
     float target_velocity = 0.f;
 
-    if (p && time_stunned == 0.f) {
-      /* Player is on the right */
-      if (p->get_pos ().x > get_pos ().x)
-        target_velocity = walk_speed;
-      else /* player in on the left */
-        target_velocity = (-1.f) * walk_speed;
+    if (stomped_timer.get_timeleft() >= 0.05f)
+    {
+      target_velocity = 0.f;
+    }
+    else if (p && time_stunned == 0.0f)
+    {
+      /* Player is on the right or left*/
+      target_velocity = (p->get_pos().x > get_pos().x) ? walk_speed : (-1.f) * walk_speed;
     }
 
-    WalkingBadguy::active_update(dt_sec, target_velocity);
+    WalkingBadguy::active_update(dt_sec, target_velocity, 3.f);
   }
-  else {
+  else
     WalkingBadguy::active_update(dt_sec);
-  }
 }
 
 void
@@ -187,10 +239,8 @@ Haywire::freeze() {
 void
 Haywire::start_exploding()
 {
-  set_action ((m_dir == Direction::LEFT) ? "ticking-left" : "ticking-right", /* loops = */ -1);
-  walk_left_action = "ticking-left";
-  walk_right_action = "ticking-right";
   set_walk_speed (EXPLODING_WALK_SPEED);
+  max_drop_height = -1;
   time_until_explosion = TIME_EXPLOSION;
   is_exploding = true;
 
@@ -212,6 +262,7 @@ Haywire::stop_exploding()
   walk_left_action = "left";
   walk_right_action = "right";
   set_walk_speed(NORMAL_WALK_SPEED);
+  max_drop_height = 16;
   time_until_explosion = 0.0f;
   is_exploding = false;
 
@@ -244,4 +295,18 @@ void Haywire::play_looping_sounds()
   }
 }
 
+HitResponse Haywire::collision_badguy(BadGuy& badguy, const CollisionHit& hit)
+{
+  if (is_exploding)
+  {
+    badguy.kill_fall();
+    return FORCE_MOVE;
+  }
+  else
+  {
+    WalkingBadguy::collision_badguy(badguy, hit);
+  }
+  return ABORT_MOVE;
+}
+
 /* EOF */
