@@ -24,7 +24,7 @@ SnowBall::SnowBall(const ReaderMapping& reader)
   walk_speed = 80;
 }
 
-SnowBall::SnowBall(const Vector& pos, Direction d, std::string script)
+SnowBall::SnowBall(const Vector& pos, Direction d, const std::string& script)
   : WalkingBadguy(pos, d, "images/creatures/snowball/snowball.sprite", "left", "right")
 {
   walk_speed = 80;
