@@ -21,8 +21,8 @@
 #include <algorithm>
 
 #include "collision/collision_hit.hpp"
+#include "math/fwd.hpp"
 
-class Vector;
 class Rectf;
 class AATriangle;
 
@@ -32,16 +32,11 @@ class Constraints final
 {
 public:
   Constraints() :
-    ground_movement(),
     hit(),
     position_left(),
     position_right(),
     position_top(),
-    position_bottom(),
-    speed_left(),
-    speed_right(),
-    speed_top(),
-    speed_bottom()
+    position_bottom()
   {
     float infinity = (std::numeric_limits<float>::has_infinity ?
                       std::numeric_limits<float>::infinity() :
@@ -50,11 +45,6 @@ public:
     position_right = infinity;
     position_top = -infinity;
     position_bottom = infinity;
-
-    speed_left = -infinity;
-    speed_right = infinity;
-    speed_top = -infinity;
-    speed_bottom = infinity;
   }
 
   bool has_constraints() const
@@ -71,30 +61,28 @@ public:
 
 public:
 
-  void constrain_left (float position, float velocity)
+  void constrain_left (float position)
   {
     position_left = std::max (position_left, position);
-    speed_left = std::max (speed_left, velocity);
   }
 
-  void constrain_right (float position, float velocity)
+  void constrain_right (float position)
   {
     position_right = std::min (position_right, position);
-    speed_right = std::min (speed_right, velocity);
   }
 
-  void constrain_top (float position, float velocity)
+  void constrain_top (float position)
   {
     position_top = std::max (position_top, position);
-    speed_top = std::max (speed_top, velocity);
   }
 
-  void constrain_bottom (float position, float velocity)
+  void constrain_bottom (float position)
   {
     position_bottom = std::min (position_bottom, position);
-    speed_bottom = std::min (speed_bottom, velocity);
   }
 
+  void merge_constraints (const Constraints& other);
+
   float get_position_left   () const { return position_left;   }
   float get_position_right  () const { return position_right;  }
   float get_position_top    () const { return position_top;    }
@@ -105,7 +93,6 @@ public:
 
   float get_x_midpoint () const { return (.5f * (position_left + position_right)); }
 
-  Vector ground_movement;
   CollisionHit hit;
 
 private:
@@ -113,11 +100,6 @@ private:
   float position_right;
   float position_top;
   float position_bottom;
-
-  float speed_left;
-  float speed_right;
-  float speed_top;
-  float speed_bottom;
 };
 
 /** checks if 2 rectangle intersect each other */
@@ -127,10 +109,13 @@ bool intersects(const Rectf& r1, const Rectf& r2);
  * Returns true in case of a collision and fills in the hit structure then.
  */
 bool rectangle_aatriangle(Constraints* constraints, const Rectf& rect,
-                          const AATriangle& triangle, const Vector& addl_ground_movement = Vector(0,0));
+                          const AATriangle& triangle);
 
-void set_rectangle_rectangle_constraints(Constraints* constraints,
-                                         const Rectf& r1, const Rectf& r2, const Vector& addl_ground_movement = Vector(0,0));
+bool rectangle_aatriangle(Constraints* constraints, const Rectf& rect,
+                          const AATriangle& triangle,
+                          bool& hits_rectangle_bottom);
+
+void set_rectangle_rectangle_constraints(Constraints* constraints, const Rectf& r1, const Rectf& r2);
 
 bool line_intersects_line(const Vector& line1_start, const Vector& line1_end, const Vector& line2_start, const Vector& line2_end);
 bool intersects_line(const Rectf& r, const Vector& line_start, const Vector& line_end);
