@@ -33,9 +33,13 @@ public:
   virtual ~ObjectIcon();
 
   virtual void draw(DrawingContext& context, const Vector& pos);
+  virtual void draw(DrawingContext& context, const Vector& pos, int pixels_shown);
 
   std::string get_object_class() const { return m_object_class; }
 
+  ObjectIcon(const ObjectIcon&) = default;
+  ObjectIcon& operator=(const ObjectIcon&) = default;
+
 private:
   void calculate_offset();
 
