@@ -26,12 +26,14 @@ class ObjectMenu final : public Menu
 {
 public:
   enum {
-    MNID_REMOVE
+    MNID_REMOVE,
+    MNID_TEST_FROM_HERE,
+    MNID_OPEN_PARTICLE_EDITOR,
   };
 
 public:
   ObjectMenu(Editor& editor, GameObject* go);
-  ~ObjectMenu();
+  ~ObjectMenu() override;
 
   virtual void menu_action(MenuItem& item) override;
   virtual bool on_back_action() override;
