@@ -30,11 +30,17 @@
 #include "editor/worldmap_objects.hpp"
 #include "gui/menu.hpp"
 #include "gui/menu_manager.hpp"
+#include "math/bezier.hpp"
 #include "object/camera.hpp"
 #include "object/path_gameobject.hpp"
 #include "object/tilemap.hpp"
+#include "supertux/gameconfig.hpp"
+#include "supertux/autotile.hpp"
 #include "supertux/game_object_factory.hpp"
+#include "supertux/resources.hpp"
 #include "supertux/sector.hpp"
+#include "video/color.hpp"
+#include "video/drawing_context.hpp"
 #include "video/renderer.hpp"
 #include "video/video_system.hpp"
 #include "video/viewport.hpp"
@@ -45,14 +51,13 @@ namespace {
 
 } // namespace
 
-bool EditorOverlayWidget::render_background = true;
-bool EditorOverlayWidget::render_grid = true;
-bool EditorOverlayWidget::snap_to_grid = true;
-int EditorOverlayWidget::selected_snap_grid_size = 3;
+bool EditorOverlayWidget::action_pressed = false;
+bool EditorOverlayWidget::alt_pressed = false;
 
 EditorOverlayWidget::EditorOverlayWidget(Editor& editor) :
   m_editor(editor),
   m_hovered_tile(0, 0),
+  m_hovered_corner(0, 0),
   m_sector_pos(0, 0),
   m_mouse_pos(0, 0),
   m_dragging(false),
@@ -64,7 +69,8 @@ EditorOverlayWidget::EditorOverlayWidget(Editor& editor) :
   m_edited_path(nullptr),
   m_last_node_marker(nullptr),
   m_object_tip(),
-  m_obj_mouse_desync(0, 0)
+  m_obj_mouse_desync(0, 0),
+  m_rectangle_preview(new TileSelection())
 {
 }
 
@@ -114,7 +120,7 @@ EditorOverlayWidget::delete_markers()
 }
 
 Rectf
-EditorOverlayWidget::drag_rect()
+EditorOverlayWidget::drag_rect() const
 {
   int start_x, start_y, end_x, end_y;
 
@@ -158,25 +164,139 @@ EditorOverlayWidget::input_tile(const Vector& pos, uint32_t tile)
   tilemap->change(static_cast<int>(pos.x), static_cast<int>(pos.y), tile);
 }
 
+void
+EditorOverlayWidget::autotile(const Vector& pos, uint32_t tile)
+{
+  auto tilemap = m_editor.get_selected_tilemap();
+  if (!tilemap) {
+    return;
+  }
+
+  if ( pos.x < 0 ||
+       pos.y < 0 ||
+       pos.x >= static_cast<float>(tilemap->get_width()) ||
+       pos.y >= static_cast<float>(tilemap->get_height())) {
+    return;
+  }
+
+  tilemap->autotile(static_cast<int>(pos.x), static_cast<int>(pos.y), tile);
+}
+
+void
+EditorOverlayWidget::input_autotile(const Vector& pos, uint32_t tile)
+{
+  this->input_tile(pos, tile);
+
+  float x = pos.x;
+  float y = pos.y;
+
+  this->autotile(Vector(x - 1.0f, y - 1.0f), tile);
+  this->autotile(Vector(x       , y - 1.0f), tile);
+  this->autotile(Vector(x + 1.0f, y - 1.0f), tile);
+  this->autotile(Vector(x - 1.0f, y       ), tile);
+  this->autotile(Vector(x       , y       ), tile);
+  this->autotile(Vector(x + 1.0f, y       ), tile);
+  this->autotile(Vector(x - 1.0f, y + 1.0f), tile);
+  this->autotile(Vector(x       , y + 1.0f), tile);
+  this->autotile(Vector(x + 1.0f, y + 1.0f), tile);
+}
+
+void
+EditorOverlayWidget::autotile_corner(const Vector& pos, uint32_t tile,
+                                     TileMap::AutotileCornerOperation op)
+{
+  auto tilemap = m_editor.get_selected_tilemap();
+  if (!tilemap) {
+    return;
+  }
+
+  if ( pos.x < 0 ||
+       pos.y < 0 ||
+       pos.x >= static_cast<float>(tilemap->get_width()) ||
+       pos.y >= static_cast<float>(tilemap->get_height())) {
+    return;
+  }
+
+  tilemap->autotile_corner(static_cast<int>(pos.x), static_cast<int>(pos.y), tile, op);
+}
+
+void
+EditorOverlayWidget::input_autotile_corner(const Vector& corner, uint32_t tile, const Vector& override_pos)
+{
+  // Erase the tile - the autotiling will add the necessary tile after
+  //if (override_pos != Vector(-1.f, -1.f))
+  //  this->input_tile(override_pos, 0);
+
+  float x = corner.x;
+  float y = corner.y;
+
+  this->autotile_corner(Vector(x - 1.0f, y - 1.0f), tile,
+                        TileMap::AutotileCornerOperation::ADD_BOTTOM_RIGHT);
+  this->autotile_corner(Vector(x       , y - 1.0f), tile,
+                        TileMap::AutotileCornerOperation::ADD_BOTTOM_LEFT);
+  this->autotile_corner(Vector(x - 1.0f, y       ), tile,
+                        TileMap::AutotileCornerOperation::ADD_TOP_RIGHT);
+  this->autotile_corner(Vector(x       , y       ), tile,
+                        TileMap::AutotileCornerOperation::ADD_TOP_LEFT);
+}
+
 void
 EditorOverlayWidget::put_tile()
 {
   auto tiles = m_editor.get_tiles();
-  Vector add_tile;
+  Vector add_tile(0.0f, 0.0f);
   for (add_tile.x = static_cast<float>(tiles->m_width) - 1.0f; add_tile.x >= 0.0f; add_tile.x--) {
     for (add_tile.y = static_cast<float>(tiles->m_height) - 1.0f; add_tile.y >= 0; add_tile.y--) {
-      input_tile(m_hovered_tile + add_tile, tiles->pos(static_cast<int>(add_tile.x),
-                                                     static_cast<int>(add_tile.y)));
+
+      uint32_t tile = tiles->pos(static_cast<int>(add_tile.x), static_cast<int>(add_tile.y));
+      auto tilemap = m_editor.get_selected_tilemap();
+
+      if (g_config->editor_autotile_mode && ((tilemap && tilemap->get_autotileset(tile)) || tile == 0)) {
+        if (tile == 0) {
+          tilemap->autotile_erase(m_hovered_tile + add_tile, m_hovered_corner + add_tile);
+        } else if (tilemap->get_autotileset(tile)->is_corner()) {
+          input_autotile_corner(m_hovered_corner + add_tile,
+                                tile,
+                                m_hovered_tile + add_tile);
+        } else {
+          input_autotile(m_hovered_tile + add_tile, tile);
+        }
+      } else {
+        input_tile(m_hovered_tile + add_tile, tile);
+      }
+
+    } // for tile y
+  } // for tile x
+}
+
+void
+EditorOverlayWidget::preview_rectangle()
+{
+  Rectf dr = drag_rect();
+  dr.set_p1(glm::floor(sp_to_tp(dr.p1())));
+  dr.set_p2(glm::floor(sp_to_tp(dr.p2())));
+  bool sgn_x = m_drag_start.x < m_sector_pos.x;
+  bool sgn_y = m_drag_start.y < m_sector_pos.y;
+
+  m_rectangle_preview->m_tiles.clear();
+  m_rectangle_preview->m_width = static_cast<int>(dr.get_width()) + 1;
+  m_rectangle_preview->m_height = static_cast<int>(dr.get_height()) + 1;
+  int y_ = sgn_y ? 0 : static_cast<int>(-dr.get_height());
+  for (int y = static_cast<int>(dr.get_top()); y <= static_cast<int>(dr.get_bottom()); y++, y_++) {
+    int x_ = sgn_x ? 0 : static_cast<int>(-dr.get_width());
+    for (int x = static_cast<int>(dr.get_left()); x <= static_cast<int>(dr.get_right()); x++, x_++) {
+      m_rectangle_preview->m_tiles.push_back(m_editor.get_tiles()->pos(x_, y_));
     }
   }
 }
 
+
 void
 EditorOverlayWidget::draw_rectangle()
 {
   Rectf dr = drag_rect();
-  dr.set_p1(sp_to_tp(dr.p1()));
-  dr.set_p2(sp_to_tp(dr.p2()));
+  dr.set_p1(glm::floor(sp_to_tp(dr.p1())));
+  dr.set_p2(glm::floor(sp_to_tp(dr.p2())));
   bool sgn_x = m_drag_start.x < m_sector_pos.x;
   bool sgn_y = m_drag_start.y < m_sector_pos.y;
 
@@ -184,11 +304,30 @@ EditorOverlayWidget::draw_rectangle()
   for (int x = static_cast<int>(dr.get_left()); x <= static_cast<int>(dr.get_right()); x++, x_++) {
     int y_ = sgn_y ? 0 : static_cast<int>(-dr.get_height());
     for (int y = static_cast<int>(dr.get_top()); y <= static_cast<int>(dr.get_bottom()); y++, y_++) {
-      input_tile( Vector(static_cast<float>(x), static_cast<float>(y)), m_editor.get_tiles()->pos(x_, y_) );
+      if (g_config->editor_autotile_mode) {
+        input_autotile( Vector(static_cast<float>(x), static_cast<float>(y)), m_editor.get_tiles()->pos(x_, y_) );
+      } else {
+        input_tile( Vector(static_cast<float>(x), static_cast<float>(y)), m_editor.get_tiles()->pos(x_, y_) );
+      }
     }
   }
 }
 
+bool
+EditorOverlayWidget::check_tiles_for_fill(uint32_t replace_tile,
+                                          uint32_t target_tile,
+                                          uint32_t third_tile) const
+{
+  if (g_config->editor_autotile_mode) {
+    return m_editor.get_tileset()->get_autotileset_from_tile(replace_tile)
+        == m_editor.get_tileset()->get_autotileset_from_tile(target_tile)
+      && m_editor.get_tileset()->get_autotileset_from_tile(replace_tile)
+        != m_editor.get_tileset()->get_autotileset_from_tile(third_tile);
+  } else {
+    return replace_tile == target_tile && replace_tile != third_tile;
+  }
+}
+
 void
 EditorOverlayWidget::fill()
 {
@@ -231,14 +370,17 @@ EditorOverlayWidget::fill()
       continue;
     }
 
+    // Autotile will happen later, so that directional filling works properly
     input_tile(pos, tiles->pos(static_cast<int>(tpos.x), static_cast<int>(tpos.y)));
-    Vector pos_;
+
+    Vector pos_(0.0f, 0.0f);
 
     // Going left...
     pos_ = pos + Vector(-1, 0);
     if (pos_.x >= 0) {
-      if (replace_tile == tilemap->get_tile_id(static_cast<int>(pos_.x), static_cast<int>(pos_.y)) &&
-          replace_tile != tiles->pos(static_cast<int>(tpos.x - 1), static_cast<int>(tpos.y))) {
+      if (check_tiles_for_fill(replace_tile,
+          tilemap->get_tile_id(static_cast<int>(pos_.x), static_cast<int>(pos_.y)),
+          tiles->pos(static_cast<int>(tpos.x - 1), static_cast<int>(tpos.y)))) {
         pos_stack.push_back( pos_ );
         continue;
       }
@@ -247,8 +389,9 @@ EditorOverlayWidget::fill()
     // Going right...
     pos_ = pos + Vector(1, 0);
     if (pos_.x < static_cast<float>(tilemap->get_width())) {
-      if (replace_tile == tilemap->get_tile_id(static_cast<int>(pos_.x), static_cast<int>(pos_.y)) &&
-          replace_tile != tiles->pos(static_cast<int>(tpos.x + 1), static_cast<int>(tpos.y))) {
+      if (check_tiles_for_fill(replace_tile,
+          tilemap->get_tile_id(static_cast<int>(pos_.x), static_cast<int>(pos_.y)),
+          tiles->pos(static_cast<int>(tpos.x + 1), static_cast<int>(tpos.y)))) {
         pos_stack.push_back( pos_ );
         continue;
       }
@@ -257,8 +400,9 @@ EditorOverlayWidget::fill()
     // Going up...
     pos_ = pos + Vector(0, -1);
     if (pos_.y >= 0) {
-      if (replace_tile == tilemap->get_tile_id(static_cast<int>(pos_.x), static_cast<int>(pos_.y))&&
-          replace_tile != tiles->pos(static_cast<int>(tpos.x), static_cast<int>(tpos.y - 1))) {
+      if (check_tiles_for_fill(replace_tile,
+          tilemap->get_tile_id(static_cast<int>(pos_.x), static_cast<int>(pos_.y)),
+          tiles->pos(static_cast<int>(tpos.x), static_cast<int>(tpos.y - 1)))) {
         pos_stack.push_back( pos_ );
         continue;
       }
@@ -267,13 +411,19 @@ EditorOverlayWidget::fill()
     // Going down...
     pos_ = pos + Vector(0, 1);
     if (pos_.y < static_cast<float>(tilemap->get_height())) {
-      if (replace_tile == tilemap->get_tile_id(static_cast<int>(pos_.x), static_cast<int>(pos_.y)) &&
-          replace_tile != tiles->pos(static_cast<int>(tpos.x), static_cast<int>(tpos.y + 1))) {
+      if (check_tiles_for_fill(replace_tile,
+          tilemap->get_tile_id(static_cast<int>(pos_.x), static_cast<int>(pos_.y)),
+          tiles->pos(static_cast<int>(tpos.x), static_cast<int>(tpos.y + 1)))) {
         pos_stack.push_back( pos_ );
         continue;
       }
     }
 
+    // Autotile happens after directional detection (because of borders; see snow tileset)
+    if (g_config->editor_autotile_mode) {
+      input_autotile(pos, tiles->pos(static_cast<int>(tpos.x), static_cast<int>(tpos.y)));
+    }
+
     // When tiles on each side are already filled or occupied by another tiles, it ends.
     pos_stack.pop_back();
   }
@@ -282,25 +432,71 @@ EditorOverlayWidget::fill()
 void
 EditorOverlayWidget::hover_object()
 {
+  m_object_tip = nullptr;
+  m_hovered_object = nullptr;
+
+  BezierMarker* marker_hovered_without_ctrl = nullptr;
+
+  bool cache_is_marker = false;
+  int cache_layer = -2147483648;
+
   for (auto& moving_object : m_editor.get_sector()->get_objects_by_type<MovingObject>())
   {
     Rectf bbox = moving_object.get_bbox();
     if (bbox.contains(m_sector_pos)) {
       if (&moving_object != m_hovered_object) {
-        m_hovered_object = &moving_object;
-        if (moving_object.has_settings()) {
-          m_object_tip = std::make_unique<Tip>(moving_object);
+
+        // Ignore BezierMarkers if ctrl isn't pressed... (1/2)
+        auto* bezier_marker = dynamic_cast<BezierMarker*>(&moving_object);
+        if (bezier_marker)
+        {
+          if (!action_pressed)
+          {
+            marker_hovered_without_ctrl = bezier_marker;
+            continue;
+          }
+          else
+          {
+            cache_is_marker = true;
+            cache_layer = 2147483647;
+            m_hovered_object = &moving_object;
+          }
+        }
+
+        // Pick objects in this priority:
+        //   1. Markers
+        //   2. Objects with a higher layer ID
+        //   3. If many objects are on the highest layer, pick the last created one
+        //      (Which will be the one rendererd on top)
+
+        bool is_marker = static_cast<bool>(dynamic_cast<MarkerObject*>(&moving_object));
+        // The "=" part of ">=" ensures that for equal layer, the last object is picked; don't remove the "="!
+        if ((is_marker && !cache_is_marker) || moving_object.get_layer() >= cache_layer)
+        {
+          cache_is_marker = is_marker;
+          cache_layer = moving_object.get_layer();
+          m_hovered_object = &moving_object;
         }
       }
-      return;
     }
   }
-  m_object_tip = nullptr;
-  m_hovered_object = nullptr;
+
+  if (m_hovered_object && m_hovered_object->has_settings()) {
+    m_object_tip = std::make_unique<Tip>(*m_hovered_object);
+  }
+
+  // (2/2) ...but select them anyways if they weren't hovering a node marker
+  if (marker_hovered_without_ctrl && !m_hovered_object)
+  {
+    m_hovered_object = marker_hovered_without_ctrl;
+    // TODO: Temporarily disabled during ongoing discussion
+    //m_object_tip = std::make_unique<Tip>(_("Press ALT to make Bezier handles continuous"));
+    return;
+  }
 }
 
 void
-EditorOverlayWidget::edit_path(Path* path, GameObject* new_marked_object)
+EditorOverlayWidget::edit_path(PathGameObject* path, GameObject* new_marked_object)
 {
   if (!path) return;
   delete_markers();
@@ -310,12 +506,22 @@ EditorOverlayWidget::edit_path(Path* path, GameObject* new_marked_object)
     return;
   }
   m_edited_path = path;
-  m_edited_path->edit_path();
+  m_edited_path->get_path().edit_path();
   if (new_marked_object) {
     m_selected_object = new_marked_object;
   }
 }
 
+void
+EditorOverlayWidget::reset_action_press()
+{
+  if (action_pressed)
+  {
+    g_config->editor_autotile_mode = !g_config->editor_autotile_mode;
+    action_pressed = false;
+  }
+}
+
 void
 EditorOverlayWidget::select_object()
 {
@@ -328,10 +534,10 @@ EditorOverlayWidget::select_object()
     return;
   }
 
-  auto path_obj = dynamic_cast<PathObject*>(m_dragged_object);
-  if (path_obj && path_obj->get_path())
+  auto path_obj = dynamic_cast<PathObject*>(m_dragged_object.get());
+  if (path_obj && path_obj->get_path_gameobject())
   {
-    edit_path(path_obj->get_path(), m_dragged_object);
+    edit_path(path_obj->get_path_gameobject(), m_dragged_object.get());
   }
 }
 
@@ -348,7 +554,7 @@ EditorOverlayWidget::grab_object()
       m_dragged_object = m_hovered_object;
       m_obj_mouse_desync = m_sector_pos - m_hovered_object->get_pos();
 
-      auto* pm = dynamic_cast<MarkerObject*>(m_hovered_object);
+      auto* pm = dynamic_cast<MarkerObject*>(m_hovered_object.get());
       if (!pm) {
         select_object();
       }
@@ -379,7 +585,7 @@ EditorOverlayWidget::clone_object()
       return;
     }
 
-    auto* pm = dynamic_cast<MarkerObject*>(m_hovered_object);
+    auto* pm = dynamic_cast<MarkerObject*>(m_hovered_object.get());
     if (!pm)
     {
       m_obj_mouse_desync = m_sector_pos - m_hovered_object->get_pos();
@@ -427,15 +633,30 @@ EditorOverlayWidget::move_object()
       return;
     }
     Vector new_pos = m_sector_pos - m_obj_mouse_desync;
-    if (snap_to_grid) {
-      auto& snap_grid_size = snap_grid_sizes[selected_snap_grid_size];
-      new_pos = (new_pos / static_cast<float>(snap_grid_size)).floor() * static_cast<float>(snap_grid_size);
+    if (g_config->editor_snap_to_grid) {
+      auto& snap_grid_size = snap_grid_sizes[g_config->editor_selected_snap_grid_size];
+      new_pos = glm::floor(new_pos / static_cast<float>(snap_grid_size)) * static_cast<float>(snap_grid_size);
 
-      auto pm = dynamic_cast<MarkerObject*>(m_dragged_object);
+      auto pm = dynamic_cast<MarkerObject*>(m_dragged_object.get());
       if (pm) {
         new_pos -= pm->get_offset();
       }
     }
+
+    // TODO: Temporarily disabled during ongoing discussion
+    // Special case: Bezier markers should influence each other when holding shift
+    //if (alt_pressed) {
+    //  auto bm = dynamic_cast<BezierMarker*>(m_dragged_object);
+    //  if (bm) {
+    //    auto nm = bm->get_parent();
+    //    if (nm) {
+    //      nm->move_other_marker(bm->get_uid(), nm->get_pos() * 2.f - new_pos);
+    //    } else {
+    //      log_warning << "Moving bezier handles without parent NodeMarker" << std::endl;
+    //    }
+    //  }
+    //}
+
     m_dragged_object->move_to(new_pos);
   }
 }
@@ -486,12 +707,23 @@ EditorOverlayWidget::add_path_node()
 {
   Path::Node new_node;
   new_node.position = m_sector_pos;
+  new_node.bezier_before = new_node.position;
+  new_node.bezier_after = new_node.position;
   new_node.time = 1;
-  m_edited_path->m_nodes.insert(m_last_node_marker->m_node + 1, new_node);
-  Sector::get().add<NodeMarker>(m_edited_path, m_edited_path->m_nodes.end() - 1, m_edited_path->m_nodes.size() - 1);
+  m_edited_path->get_path().m_nodes.insert(m_last_node_marker->m_node + 1, new_node);
+  auto& bezier_before = Sector::get().add<BezierMarker>(&(*(m_edited_path->get_path().m_nodes.end() - 1)), &((m_edited_path->get_path().m_nodes.end() - 1)->bezier_before));
+  auto& bezier_after = Sector::get().add<BezierMarker>(&(*(m_edited_path->get_path().m_nodes.end() - 1)), &((m_edited_path->get_path().m_nodes.end() - 1)->bezier_after));
+  auto& new_marker = Sector::get().add<NodeMarker>(&(m_edited_path.get()->get_path()), m_edited_path->get_path().m_nodes.end() - 1, m_edited_path->get_path().m_nodes.size() - 1, bezier_before.get_uid(), bezier_after.get_uid());
+  bezier_before.set_parent(new_marker.get_uid());
+  bezier_after.set_parent(new_marker.get_uid());
   //last_node_marker = dynamic_cast<NodeMarker*>(marker.get());
   update_node_iterators();
+  new_marker.update_node_times();
   m_editor.get_sector()->flush_game_objects();
+
+  // This will ensure that we will hover NodeMarkers in priority before BezierMarkers
+  hover_object();
+
   grab_object();
 }
 
@@ -510,10 +742,10 @@ EditorOverlayWidget::put_object()
   else
   {
     auto target_pos = m_sector_pos;
-    if (snap_to_grid)
+    if (g_config->editor_snap_to_grid)
     {
-      auto& snap_grid_size = snap_grid_sizes[selected_snap_grid_size];
-      target_pos = (m_sector_pos / static_cast<float>(snap_grid_size)).floor() * static_cast<float>(snap_grid_size);
+      auto& snap_grid_size = snap_grid_sizes[g_config->editor_selected_snap_grid_size];
+      target_pos = glm::floor(m_sector_pos / static_cast<float>(snap_grid_size)) * static_cast<float>(snap_grid_size);
     }
 
     auto object = GameObjectFactory::instance().create(object_class, target_pos, Direction::LEFT);
@@ -524,14 +756,14 @@ EditorOverlayWidget::put_object()
       if (!dynamic_cast<PathGameObject*>(object.get())) {
         m_editor.add_layer(object.get());
       }
-    } else if (!snap_to_grid) {
+    } else if (!g_config->editor_snap_to_grid) {
       auto bbox = mo->get_bbox();
       mo->move_to(mo->get_pos() - Vector(bbox.get_width() / 2, bbox.get_height() / 2));
     }
 
     auto* wo = dynamic_cast<worldmap_editor::WorldmapObject*>(object.get());
     if (wo) {
-      wo->move_to(wo->get_pos() / 32);
+      wo->move_to(wo->get_pos() / 32.0f);
     }
 
     m_editor.get_sector()->add_object(std::move(object));
@@ -541,6 +773,8 @@ EditorOverlayWidget::put_object()
 void
 EditorOverlayWidget::process_left_click()
 {
+  if (MenuManager::instance().has_dialog())
+    return;
   m_dragging = true;
   m_dragging_right = false;
   m_drag_start = m_sector_pos;
@@ -555,7 +789,7 @@ EditorOverlayWidget::process_left_click()
           break;
 
         case 1:
-          draw_rectangle();
+          preview_rectangle();
           break;
 
         case 2:
@@ -626,7 +860,7 @@ EditorOverlayWidget::process_right_click()
 }
 
 Rectf
-EditorOverlayWidget::tile_drag_rect()
+EditorOverlayWidget::tile_drag_rect() const
 {
   Rectf result = drag_rect();
 
@@ -642,7 +876,7 @@ EditorOverlayWidget::tile_drag_rect()
 }
 
 Rectf
-EditorOverlayWidget::selection_draw_rect()
+EditorOverlayWidget::selection_draw_rect() const
 {
   Rectf select = tile_drag_rect();
   select.set_p1(tile_screen_pos(select.p1()));
@@ -680,7 +914,23 @@ EditorOverlayWidget::update_tile_selection()
 bool
 EditorOverlayWidget::on_mouse_button_up(const SDL_MouseButtonEvent& button)
 {
+  if (button.button == SDL_BUTTON_LEFT)
+  {
+    if (m_editor.get_tileselect_input_type() == EditorToolboxWidget::InputType::TILE
+        && m_editor.get_tileselect_select_mode() == 1)
+    {
+      if (m_dragging)
+      {
+        draw_rectangle();
+        m_rectangle_preview->m_tiles.clear();
+      }
+    }
+  }
+
   m_dragging = false;
+
+  // Return true anyways, because that's how it worked when this function only
+  // had `m_dragging = false;` in its body.
   return true;
 }
 
@@ -721,7 +971,7 @@ EditorOverlayWidget::on_mouse_motion(const SDL_MouseMotionEvent& motion)
               put_tile();
               break;
             case 1:
-              draw_rectangle();
+              preview_rectangle();
               break;
             default:
               break;
@@ -756,7 +1006,19 @@ EditorOverlayWidget::on_key_up(const SDL_KeyboardEvent& key)
   auto sym = key.keysym.sym;
   if (sym == SDLK_LSHIFT || sym == SDLK_RSHIFT)
   {
-    snap_to_grid = !snap_to_grid;
+    g_config->editor_snap_to_grid = !g_config->editor_snap_to_grid;
+  }
+  if (sym == SDLK_LCTRL || sym == SDLK_RCTRL) {
+    if (action_pressed)
+    {
+      g_config->editor_autotile_mode = !g_config->editor_autotile_mode;
+      action_pressed = false;
+    }
+    // Hovered objects depend on which keys are pressed
+    hover_object();
+  }
+  if (sym == SDLK_LALT || sym == SDLK_RALT) {
+    alt_pressed = false;
   }
   return true;
 }
@@ -766,10 +1028,19 @@ EditorOverlayWidget::on_key_down(const SDL_KeyboardEvent& key)
 {
   auto sym = key.keysym.sym;
   if (sym == SDLK_F8) {
-    render_grid = !render_grid;
+    g_config->editor_render_grid = !g_config->editor_render_grid;
   }
   if (sym == SDLK_F7 || sym == SDLK_LSHIFT || sym == SDLK_RSHIFT) {
-    snap_to_grid = !snap_to_grid;
+    g_config->editor_snap_to_grid = !g_config->editor_snap_to_grid;
+  }
+  if (sym == SDLK_F5 || ((sym == SDLK_LCTRL || sym == SDLK_RCTRL) && !action_pressed)) {
+    g_config->editor_autotile_mode = !g_config->editor_autotile_mode;
+    action_pressed = true;
+    // Hovered objects depend on which keys are pressed
+    hover_object();
+  }
+  if (sym == SDLK_LALT || sym == SDLK_RALT) {
+    alt_pressed = true;
   }
   return true;
 }
@@ -782,6 +1053,9 @@ EditorOverlayWidget::update_pos()
 
   m_sector_pos = m_mouse_pos + m_editor.get_sector()->get_camera().get_translation();
   m_hovered_tile = sp_to_tp(m_sector_pos);
+
+  float half_tile = 16.f;
+  m_hovered_corner = sp_to_tp(m_sector_pos + Vector(half_tile, half_tile));
   // update tip
   hover_object();
 }
@@ -796,23 +1070,29 @@ EditorOverlayWidget::draw_tile_tip(DrawingContext& context)
       return;
     }
 
-    Vector drawn_tile = m_hovered_tile;
+    if (m_editor.get_tiles()->empty())
+      return;
+
+    Vector screen_corner = context.get_cliprect().p2() +
+                         m_editor.get_sector()->get_camera().get_translation();
+    Vector drawn_tile = m_hovered_tile; // FIXME: Why is this initialised if it's going to be overwritten right below?
     auto tiles = m_editor.get_tiles();
 
     for (drawn_tile.x = static_cast<float>(tiles->m_width) - 1.0f; drawn_tile.x >= 0.0f; drawn_tile.x--) {
       for (drawn_tile.y = static_cast<float>(tiles->m_height) - 1.0f; drawn_tile.y >= 0.0f; drawn_tile.y--) {
         Vector on_tile = m_hovered_tile + drawn_tile;
 
-        if (m_editor.get_tiles()->empty() ||
-            on_tile.x < 0 ||
+        if (on_tile.x < 0 ||
             on_tile.y < 0 ||
             on_tile.x >= static_cast<float>(tilemap->get_width()) ||
-            on_tile.y >= static_cast<float>(tilemap->get_height())) {
+            on_tile.y >= static_cast<float>(tilemap->get_height()) ||
+            on_tile.x >= ceilf(screen_corner.x / 32) ||
+            on_tile.y >= ceilf(screen_corner.y / 32)) {
           continue;
         }
         uint32_t tile_id = tiles->pos(static_cast<int>(drawn_tile.x), static_cast<int>(drawn_tile.y));
         draw_tile(context.color(), *m_editor.get_tileset(), tile_id,
-                  tp_to_sp(on_tile) - m_editor.get_sector()->get_camera().get_translation(),
+                  align_to_tilemap(on_tile) - m_editor.get_sector()->get_camera().get_translation(),
                   LAYER_GUI-11, Color(1, 1, 1, 0.5));
         /*if (tile_id) {
           const Tile* tg_tile = m_editor.get_tileset()->get( tile_id );
@@ -825,12 +1105,51 @@ EditorOverlayWidget::draw_tile_tip(DrawingContext& context)
 }
 
 void
-EditorOverlayWidget::draw_tile_grid(DrawingContext& context, const Color& line_color, int tile_size)
+EditorOverlayWidget::draw_rectangle_preview(DrawingContext& context)
+{
+  auto tilemap = m_editor.get_selected_tilemap();
+  if (!tilemap) {
+    return;
+  }
+
+  if (m_rectangle_preview->empty())
+    return;
+
+  Vector screen_corner = context.get_cliprect().p2() +
+                        m_editor.get_sector()->get_camera().get_translation();
+  Vector drawn_tile(0.0f, 0.0f);
+  Vector corner(std::min(sp_to_tp(m_drag_start).x, m_hovered_tile.x),
+                std::min(sp_to_tp(m_drag_start).y, m_hovered_tile.y));
+  auto tiles = m_rectangle_preview.get();
+
+  for (drawn_tile.x = static_cast<float>(tiles->m_width) - 1.0f; drawn_tile.x >= 0.0f; drawn_tile.x--) {
+    for (drawn_tile.y = static_cast<float>(tiles->m_height) - 1.0f; drawn_tile.y >= 0.0f; drawn_tile.y--) {
+      Vector on_tile = corner + drawn_tile;
+
+      if (on_tile.x < 0 ||
+          on_tile.y < 0 ||
+          on_tile.x >= static_cast<float>(tilemap->get_width()) ||
+          on_tile.y >= static_cast<float>(tilemap->get_height()) ||
+          on_tile.x >= ceilf(screen_corner.x / 32) ||
+          on_tile.y >= ceilf(screen_corner.y / 32)) {
+        continue;
+      }
+      uint32_t tile_id = tiles->pos(static_cast<int>(drawn_tile.x), static_cast<int>(drawn_tile.y));
+      draw_tile(context.color(), *m_editor.get_tileset(), tile_id,
+                align_to_tilemap(on_tile) - m_editor.get_sector()->get_camera().get_translation(),
+                LAYER_GUI-11, Color(1, 1, 1, 0.5));
+    }
+  }
+}
+
+void
+EditorOverlayWidget::draw_tile_grid(DrawingContext& context, int tile_size,
+  bool draw_shadow) const
 {
   auto current_tm = m_editor.get_selected_tilemap();
   if (current_tm == nullptr)
     return;
- 
+
   int tm_width = current_tm->get_width() * (32 / tile_size);
   int tm_height = current_tm->get_height() * (32 / tile_size);
   auto cam_translation = m_editor.get_sector()->get_camera().get_translation();
@@ -844,17 +1163,45 @@ EditorOverlayWidget::draw_tile_grid(DrawingContext& context, const Color& line_c
   end.x = std::min(float(tm_width), end.x);
   end.y = std::min(float(tm_height), end.y);
 
-  Vector line_start, line_end;
+  Vector line_start(0.0f, 0.0f);
+  Vector line_end(0.0f, 0.0f);
+  auto draw_line = [&](const Vector& from, const Vector& to, const Color& col)
+  {
+    context.color().draw_line(from, to, col, current_tm->get_layer());
+  };
+  if (draw_shadow) {
+    Vector viewport_scale = VideoSystem::current()->get_viewport().get_scale();
+    const Color shadow_colour(0.0f, 0.0f, 0.0f, 0.05f);
+    const Vector shadow_offset(1.0f / viewport_scale.x,
+      1.0f / viewport_scale.y);
+    for (int i = static_cast<int>(start.x); i <= static_cast<int>(end.x); i++) {
+      line_start = tile_screen_pos(Vector(static_cast<float>(i), 0.0f),
+        tile_size) + shadow_offset;
+      line_end = tile_screen_pos(Vector(static_cast<float>(i), end.y),
+        tile_size) + shadow_offset;
+      draw_line(line_start, line_end, shadow_colour);
+    }
+
+    for (int i = static_cast<int>(start.y); i <= static_cast<int>(end.y); i++) {
+      line_start = tile_screen_pos(Vector(0.0f, static_cast<float>(i)),
+        tile_size) + shadow_offset;
+      line_end = tile_screen_pos(Vector(end.x, static_cast<float>(i)),
+        tile_size) + shadow_offset;
+      draw_line(line_start, line_end, shadow_colour);
+    }
+  }
+
+  const Color line_color(1.f, 1.f, 1.f, 0.2f);
   for (int i = static_cast<int>(start.x); i <= static_cast<int>(end.x); i++) {
-    line_start = tile_screen_pos( Vector(static_cast<float>(i), 0.0f), tile_size );
-    line_end = tile_screen_pos( Vector(static_cast<float>(i), end.y), tile_size );
-    context.color().draw_line(line_start, line_end, line_color, current_tm->get_layer());
+    line_start = tile_screen_pos(Vector(static_cast<float>(i), 0.0f), tile_size);
+    line_end = tile_screen_pos(Vector(static_cast<float>(i), end.y), tile_size);
+    draw_line(line_start, line_end, line_color);
   }
 
   for (int i = static_cast<int>(start.y); i <= static_cast<int>(end.y); i++) {
-    line_start = tile_screen_pos( Vector(0.0f, static_cast<float>(i)), tile_size );
-    line_end = tile_screen_pos( Vector(end.x, static_cast<float>(i)), tile_size );
-    context.color().draw_line(line_start, line_end, line_color, current_tm->get_layer());
+    line_start = tile_screen_pos(Vector(0.0f, static_cast<float>(i)), tile_size);
+    line_end = tile_screen_pos(Vector(end.x, static_cast<float>(i)), tile_size);
+    draw_line(line_start, line_end, line_color);
   }
 }
 
@@ -884,24 +1231,46 @@ EditorOverlayWidget::draw_path(DrawingContext& context)
   if (!m_selected_object->is_valid()) return;
   if (!m_edited_path->is_valid()) return;
 
-  for (auto i = m_edited_path->m_nodes.begin(); i != m_edited_path->m_nodes.end(); ++i) {
+  for (auto i = m_edited_path->get_path().m_nodes.begin(); i != m_edited_path->get_path().m_nodes.end(); ++i) {
     auto j = i+1;
     Path::Node* node1 = &(*i);
     Path::Node* node2;
-    if (j == m_edited_path->m_nodes.end()) {
-      if (m_edited_path->m_mode == WalkMode::CIRCULAR) {
+    if (j == m_edited_path->get_path().m_nodes.end()) {
+      if (m_edited_path->get_path().m_mode == WalkMode::CIRCULAR) {
         //loop to the first node
-        node2 = &(*m_edited_path->m_nodes.begin());
+        node2 = &(*m_edited_path->get_path().m_nodes.begin());
       } else {
+        // Just draw the bezier lines
+        auto cam_translation = m_editor.get_sector()->get_camera().get_translation();
+        context.color().draw_line(node1->position - cam_translation,
+                                  node1->bezier_before - cam_translation,
+                                  Color(0, 0, 1), LAYER_GUI - 21);
+        context.color().draw_line(node1->position - cam_translation,
+                                  node1->bezier_after - cam_translation,
+                                  Color(0, 0, 1), LAYER_GUI - 21);
         continue;
       }
     } else {
       node2 = &(*j);
     }
     auto cam_translation = m_editor.get_sector()->get_camera().get_translation();
+    Bezier::draw_curve(context,
+                       node1->position - cam_translation,
+                       node1->bezier_after - cam_translation,
+                       node2->bezier_before - cam_translation,
+                       node2->position - cam_translation,
+                       100,
+                       Color::RED,
+                       LAYER_GUI - 21);
     context.color().draw_line(node1->position - cam_translation,
-                              node2->position - cam_translation,
-                              Color(1, 0, 0), LAYER_GUI - 21);
+                              node1->bezier_before - cam_translation,
+                              Color(0, 0, 1), LAYER_GUI - 21);
+    context.color().draw_line(node1->position - cam_translation,
+                              node1->bezier_after - cam_translation,
+                              Color(0, 0, 1), LAYER_GUI - 21);
+    //context.color().draw_line(node1->position - cam_translation,
+    //                          node2->position - cam_translation,
+    //                          Color(1, 0, 0), LAYER_GUI - 21);
   }
 }
 
@@ -909,14 +1278,15 @@ void
 EditorOverlayWidget::draw(DrawingContext& context)
 {
   draw_tile_tip(context);
+  draw_rectangle_preview(context);
   draw_path(context);
 
-  if (render_grid) {
-    draw_tile_grid(context, Color(1.f, 1.f, 1.f, 0.2f));
+  if (g_config->editor_render_grid) {
+    draw_tile_grid(context, 32, true);
     draw_tilemap_border(context);
-    auto snap_grid_size = snap_grid_sizes[selected_snap_grid_size];
+    auto snap_grid_size = snap_grid_sizes[g_config->editor_selected_snap_grid_size];
     if (snap_grid_size != 32) {
-      draw_tile_grid(context, Color(1.f, 1.f, 1.f, 0.2f), snap_grid_size);
+      draw_tile_grid(context, snap_grid_size, false);
     }
   }
 
@@ -929,19 +1299,26 @@ EditorOverlayWidget::draw(DrawingContext& context)
     // Draw selection rectangle...
     auto cam_translation = m_editor.get_sector()->get_camera().get_translation();
     Vector p0 = m_drag_start - cam_translation;
-    Vector p1 = Vector(m_drag_start.x, m_sector_pos.y) - cam_translation;
-    Vector p2 = Vector(m_sector_pos.x, m_drag_start.y) - cam_translation;
+    Vector p3 = m_mouse_pos;
+    if (p0.x > p3.x) {
+      std::swap(p0.x, p3.x);
+    }
+    if (p0.y > p3.y) {
+      std::swap(p0.y, p3.y);
+    }
+    Vector p1 = Vector(p0.x, p3.y);
+    Vector p2 = Vector(p3.x, p0.y);
 
     context.color().draw_filled_rect(Rectf(p0, p1 + Vector(2, 2)),
                                        Color(0.0f, 1.0f, 0.0f, 1.0f), 0.0f, LAYER_GUI-5);
-    context.color().draw_filled_rect(Rectf(p2, m_mouse_pos + Vector(2, 2)),
+    context.color().draw_filled_rect(Rectf(p2, p3 + Vector(2, 2)),
                                        Color(0.0f, 1.0f, 0.0f, 1.0f), 0.0f, LAYER_GUI-5);
     context.color().draw_filled_rect(Rectf(p0, p2 + Vector(2, 2)),
                                        Color(0.0f, 1.0f, 0.0f, 1.0f), 0.0f, LAYER_GUI-5);
-    context.color().draw_filled_rect(Rectf(p1, m_mouse_pos + Vector(2, 2)),
+    context.color().draw_filled_rect(Rectf(p1, p3 + Vector(2, 2)),
                                        Color(0.0f, 1.0f, 0.0f, 1.0f), 0.0f, LAYER_GUI-5);
 
-    context.color().draw_filled_rect(Rectf(p0, m_mouse_pos),
+    context.color().draw_filled_rect(Rectf(p0, p3),
                                        Color(0.0f, 1.0f, 0.0f, 0.2f), 0.0f, LAYER_GUI-5);
   }
 
@@ -949,10 +1326,31 @@ EditorOverlayWidget::draw(DrawingContext& context)
     context.color().draw_filled_rect(selection_draw_rect(),
                                        Color(0.2f, 0.4f, 1.0f, 0.6f), 0.0f, LAYER_GUI-13);
   }
+
+
+  if (g_config->editor_autotile_help) {
+    if (m_editor.get_tileset()->get_autotileset_from_tile(m_editor.get_tiles()->pos(0, 0)) != nullptr)
+    {
+      if (g_config->editor_autotile_mode) {
+        context.color().draw_text(Resources::normal_font, _("Autotile mode is on"), Vector(144, 16), ALIGN_LEFT, LAYER_OBJECTS+1, EditorOverlayWidget::text_autotile_active_color);
+      } else {
+        context.color().draw_text(Resources::normal_font, _("Hold Ctrl to enable autotile"), Vector(144, 16), ALIGN_LEFT, LAYER_OBJECTS+1, EditorOverlayWidget::text_autotile_available_color);
+      }
+    } else if (g_config->editor_autotile_mode) {
+      if (m_editor.get_tiles()->pos(0, 0) == 0) {
+        context.color().draw_text(Resources::normal_font, _("Autotile erasing mode is on"), Vector(144, 16), ALIGN_LEFT, LAYER_OBJECTS+1, EditorOverlayWidget::text_autotile_active_color);
+      } else {
+        context.color().draw_text(Resources::normal_font, _("Selected tile isn't autotileable"), Vector(144, 16), ALIGN_LEFT, LAYER_OBJECTS+1, EditorOverlayWidget::text_autotile_error_color);
+      }
+    } else if (m_editor.get_tiles()->pos(0, 0) == 0) {
+        context.color().draw_text(Resources::normal_font, _("Hold Ctrl to enable autotile erasing"), Vector(144, 16), ALIGN_LEFT, LAYER_OBJECTS+1, EditorOverlayWidget::text_autotile_available_color);
+    }
+  }
+
 }
 
 Vector
-EditorOverlayWidget::tp_to_sp(const Vector& tp, int tile_size)
+EditorOverlayWidget::tp_to_sp(const Vector& tp, int tile_size) const
 {
   auto tilemap = m_editor.get_selected_tilemap();
   if (!tilemap)
@@ -965,7 +1363,7 @@ EditorOverlayWidget::tp_to_sp(const Vector& tp, int tile_size)
 }
 
 Vector
-EditorOverlayWidget::sp_to_tp(const Vector& sp, int tile_size)
+EditorOverlayWidget::sp_to_tp(const Vector& sp, int tile_size) const
 {
   auto tilemap = m_editor.get_selected_tilemap();
   if (!tilemap)
@@ -978,10 +1376,23 @@ EditorOverlayWidget::sp_to_tp(const Vector& sp, int tile_size)
 }
 
 Vector
-EditorOverlayWidget::tile_screen_pos(const Vector& tp, int tile_size)
+EditorOverlayWidget::tile_screen_pos(const Vector& tp, int tile_size) const
 {
   Vector sp = tp_to_sp(tp, tile_size);
   return sp - m_editor.get_sector()->get_camera().get_translation();
 }
 
+Vector
+EditorOverlayWidget::align_to_tilemap(const Vector& sp, int tile_size) const
+{
+  auto tilemap = m_editor.get_selected_tilemap();
+  if (!tilemap)
+  {
+    return Vector(0, 0);
+  }
+
+  Vector sp_ = sp + tilemap->get_offset() / static_cast<float>(tile_size);
+  return glm::trunc(sp_) * static_cast<float>(tile_size);
+}
+
 /* EOF */
