@@ -43,6 +43,7 @@
 #include "supertux/moving_object.hpp"
 #include "scripting/ambient_sound.hpp"
 #include "squirrel/exposed_object.hpp"
+#include "video/layer.hpp"
 
 class GameObject;
 class ReaderMapping;
@@ -54,7 +55,7 @@ class AmbientSound final : public MovingObject,
 public:
   AmbientSound(const ReaderMapping& mapping);
   AmbientSound(const Vector& pos, float factor, float bias, float vol, const std::string& file);
-  ~AmbientSound();
+  ~AmbientSound() override;
 
   virtual HitResponse collision(GameObject& other, const CollisionHit& hit_) override;
 
@@ -77,6 +78,8 @@ public:
   virtual ObjectSettings get_settings() override;
   virtual void after_editor_set() override;
 
+  virtual int get_layer() const override { return LAYER_OBJECTS; }
+
 protected:
   virtual void update(float dt_sec) override;
   virtual void start_playing();
