@@ -55,10 +55,11 @@ class BicyclePlatform final : public GameObject
 
 public:
   BicyclePlatform(const ReaderMapping& reader);
-  virtual ~BicyclePlatform();
+  ~BicyclePlatform() override;
 
   virtual void draw(DrawingContext& context) override;
   virtual void update(float dt_sec) override;
+  virtual void on_flip(float height) override;
 
   virtual std::string get_class() const override { return "bicycle-platform"; }
   virtual std::string get_display_name() const override { return _("Bicycle Platform"); }
