@@ -159,9 +159,12 @@ BonusBlock::BonusBlock(const ReaderMapping& mapping) :
     }
   }
 
-  if (m_contents == Content::LIGHT) {
+  if (m_contents == Content::LIGHT || m_contents == Content::LIGHT_ON) {
     SoundManager::current()->preload("sounds/switch.ogg");
     m_lightsprite = Surface::from_file("/images/objects/lightmap_light/bonusblock_light.png");
+    if (m_contents == Content::LIGHT_ON) {
+      m_sprite->set_action("on");
+    }
   }
 }
 
@@ -185,6 +188,7 @@ BonusBlock::get_content_by_data(int tile_data) const
     case 12: return Content::CUSTOM; // Red potion
     case 13: return Content::AIRGROW;
     case 14: return Content::EARTHGROW;
+    case 15: return Content::LIGHT_ON;
     default:
       log_warning << "Invalid box contents" << std::endl;
       return Content::COIN;
@@ -204,10 +208,10 @@ BonusBlock::get_settings()
   result.add_int(_("Count"), &m_hit_counter, "count", 1);
   result.add_enum(_("Content"), reinterpret_cast<int*>(&m_contents),
                   {_("Coin"), _("Growth (fire flower)"), _("Growth (ice flower)"), _("Growth (air flower)"),
-                   _("Growth (earth flower)"), _("Star"), _("Tux doll"), _("Custom"), _("Script"), _("Light"),
+                   _("Growth (earth flower)"), _("Star"), _("Tux doll"), _("Custom"), _("Script"), _("Light"), _("Light (On)"),
                    _("Trampoline"), _("Coin rain"), _("Coin explosion")},
                   {"coin", "firegrow", "icegrow", "airgrow", "earthgrow", "star",
-                   "1up", "custom", "script", "light", "trampoline", "rain", "explode"},
+                   "1up", "custom", "script", "light", "light-on", "trampoline", "rain", "explode"},
                   static_cast<int>(Content::COIN), "contents");
   result.add_sexp(_("Custom Content"), "custom-contents", m_custom_sx);
 
@@ -228,8 +232,11 @@ BonusBlock::collision(GameObject& other, const CollisionHit& hit_)
 {
   auto player = dynamic_cast<Player*> (&other);
   if (player) {
-    if (player->m_does_buttjump)
+    if (player->m_does_buttjump ||
+      (player->is_swimboosting() && player->get_bbox().get_bottom() < m_col.m_bbox.get_top() + SHIFT_DELTA))
+    {
       try_drop(player);
+    }
   }
 
   auto badguy = dynamic_cast<BadGuy*> (&other);
@@ -273,7 +280,7 @@ BonusBlock::try_open(Player* player)
       Sector::get().add<BouncyCoin>(get_pos(), true);
       player->get_status().add_coins(1);
       if (m_hit_counter != 0)
-        Sector::get().get_level().m_stats.m_coins++;
+        Sector::get().get_level().m_stats.increment_coins();
       break;
     }
 
@@ -326,6 +333,7 @@ BonusBlock::try_open(Player* player)
     { break; } // because scripts always run, this prevents default contents from being assumed
 
     case Content::LIGHT:
+    case Content::LIGHT_ON:
     {
       if (m_sprite->get_action() == "on")
         m_sprite->set_action("off");
@@ -364,7 +372,7 @@ BonusBlock::try_open(Player* player)
   }
 
   start_bounce(player);
-  if (m_hit_counter <= 0 || m_contents == Content::LIGHT) { //use 0 to allow infinite hits
+  if (m_hit_counter <= 0 || m_contents == Content::LIGHT || m_contents == Content::LIGHT_ON) { //use 0 to allow infinite hits
   } else if (m_hit_counter == 1) {
     m_sprite->set_action("empty");
   } else {
@@ -412,25 +420,25 @@ BonusBlock::try_drop(Player *player)
 
     case Content::FIREGROW:
     {
-      drop_growup_bonus("images/powerups/fireflower/fireflower.sprite", countdown);
+      drop_growup_bonus(player, "images/powerups/fireflower/fireflower.sprite", direction, countdown);
       break;
     }
 
     case Content::ICEGROW:
     {
-      drop_growup_bonus("images/powerups/iceflower/iceflower.sprite", countdown);
+      drop_growup_bonus(player, "images/powerups/iceflower/iceflower.sprite", direction, countdown);
       break;
     }
 
     case Content::AIRGROW:
     {
-      drop_growup_bonus("images/powerups/airflower/airflower.sprite", countdown);
+      drop_growup_bonus(player, "images/powerups/airflower/airflower.sprite", direction, countdown);
       break;
     }
 
     case Content::EARTHGROW:
     {
-      drop_growup_bonus("images/powerups/earthflower/earthflower.sprite", countdown);
+      drop_growup_bonus(player, "images/powerups/earthflower/earthflower.sprite", direction, countdown);
       break;
     }
 
@@ -467,6 +475,7 @@ BonusBlock::try_drop(Player *player)
     } // because scripts always run, this prevents default contents from being assumed
 
     case Content::LIGHT:
+    case Content::LIGHT_ON:
     case Content::TRAMPOLINE:
     case Content::RAIN:
     {
@@ -503,9 +512,12 @@ void
 BonusBlock::raise_growup_bonus(Player* player, const BonusType& bonus, const Direction& dir)
 {
   std::unique_ptr<MovingObject> obj;
-  if (player->get_status().bonus == NO_BONUS) {
-    obj = std::make_unique<GrowUp>(dir);
-  } else {
+  if (player->get_status().bonus == NO_BONUS)
+  {
+    obj = std::make_unique<GrowUp>(get_pos(), dir);
+  }
+  else
+  {
     obj = std::make_unique<Flower>(bonus);
   }
 
@@ -514,9 +526,16 @@ BonusBlock::raise_growup_bonus(Player* player, const BonusType& bonus, const Dir
 }
 
 void
-BonusBlock::drop_growup_bonus(const std::string& bonus_sprite_name, bool& countdown)
+BonusBlock::drop_growup_bonus(Player* player, const std::string& bonus_sprite_name, const Direction& dir, bool& countdown)
 {
-  Sector::get().add<PowerUp>(get_pos() + Vector(0, 32), bonus_sprite_name);
+  if (player->get_status().bonus == NO_BONUS)
+  {
+    Sector::get().add<GrowUp>(get_pos() + Vector(0, 32), dir);
+  }
+  else
+  {
+    Sector::get().add<PowerUp>(get_pos() + Vector(0, 32), bonus_sprite_name);
+  }
   SoundManager::current()->play("sounds/upgrade.wav", upgrade_sound_gain);
   countdown = true;
 }
@@ -557,6 +576,8 @@ BonusBlock::get_content_from_string(const std::string& contentstring) const
     return Content::SCRIPT;
   } else if (contentstring == "light") {
     return Content::LIGHT;
+  } else if (contentstring == "light-on") {
+    return Content::LIGHT_ON;
   } else if (contentstring == "trampoline") {
     return Content::TRAMPOLINE;
   } else if (contentstring == "rain") {
@@ -584,6 +605,7 @@ BonusBlock::contents_to_string(const BonusBlock::Content& content) const
     case Content::CUSTOM: return "custom";
     case Content::SCRIPT: return "script";
     case Content::LIGHT: return "light";
+    case Content::LIGHT_ON: return "light-on";
     case Content::TRAMPOLINE: return "trampoline";
     case Content::RAIN: return "rain";
     case Content::EXPLODE: return "explode";
@@ -597,6 +619,7 @@ BonusBlock::preload_contents(int d)
   switch (d)
   {
     case 6: // Light
+    case 15: // Light (On)
       SoundManager::current()->preload("sounds/switch.ogg");
       m_lightsprite=Surface::from_file("/images/objects/lightmap_light/bonusblock_light.png");
       break;
