@@ -18,9 +18,9 @@
 
 #include "audio/sound_manager.hpp"
 #include "badguy/badguy.hpp"
+#include "badguy/icecrusher.hpp"
 #include "object/bouncy_coin.hpp"
 #include "object/explosion.hpp"
-#include "object/icecrusher.hpp"
 #include "object/player.hpp"
 #include "object/portable.hpp"
 #include "sprite/sprite.hpp"
