@@ -34,6 +34,9 @@ public:
   virtual void update(float dt_sec) override;
   virtual void draw(DrawingContext& context) override;
   virtual HitResponse collision(GameObject& other, const CollisionHit& hit) override;
+  virtual void on_flip(float height) override;
+
+  virtual int get_layer() const override { return LAYER_OBJECTS; }
 
 private:
   BonusType type;
