@@ -27,7 +27,7 @@ public:
 
   virtual HitResponse collision(GameObject& other, const CollisionHit& hit) override;
   virtual std::string get_class() const override { return "hurting_platform"; }
-  virtual std::string get_display_name() const override { return _("Hurting platform"); }
+  virtual std::string get_display_name() const override { return _("Hurting Platform"); }
 
 private:
   HurtingPlatform(const HurtingPlatform&) = delete;
