@@ -141,7 +141,7 @@ void
 PowerUp::update(float dt_sec)
 {
   if (!no_physics)
-    m_col.m_movement = physic.get_movement(dt_sec);
+    m_col.set_movement(physic.get_movement(dt_sec));
   //Stars sparkle when close to Tux
   if (m_sprite_name == "images/powerups/star/star.sprite" || m_sprite_name == "/images/powerups/star/star.sprite"){
     if (auto* player = Sector::get().get_nearest_player(m_col.m_bbox)) {
@@ -156,7 +156,7 @@ PowerUp::update(float dt_sec)
           Vector pspeed = Vector(0, 0);
           Vector paccel = Vector(0, 0);
           Sector::get().add<SpriteParticle>(
-            "images/objects/particles/sparkle.sprite",
+            "images/particles/sparkle.sprite",
             // draw bright sparkles when very close to Tux, dark sparkles when slightly further
             (disp_x*disp_x + disp_y*disp_y <= 128*128) ?
             // make every other a longer sparkle to make trail a bit fuzzy
