@@ -21,18 +21,21 @@
 #include "object/coin.hpp"
 #include "supertux/sector.hpp"
 #include "supertux/tile.hpp"
+#include "object/player.hpp"
 #include "util/reader_mapping.hpp"
 
 namespace {
 const std::string ROCK_SOUND = "sounds/brick.wav"; //TODO use own sound.
 }
 
+static const float GROUND_FRICTION = 0.1f; // Amount of friction to apply while on ground.
+
 Rock::Rock(const Vector& pos, const std::string& spritename) :
   MovingSprite(pos, spritename),
   ExposedObject<Rock, scripting::Rock>(this),
   physic(),
   on_ground(false),
-  last_movement(),
+  last_movement(0.0f, 0.0f),
   on_grab_script(),
   on_ungrab_script()
 {
@@ -45,7 +48,7 @@ Rock::Rock(const ReaderMapping& reader) :
   ExposedObject<Rock, scripting::Rock>(this),
   physic(),
   on_ground(false),
-  last_movement(),
+  last_movement(0.0f, 0.0f),
   on_grab_script(),
   on_ungrab_script()
 {
@@ -60,7 +63,7 @@ Rock::Rock(const ReaderMapping& reader, const std::string& spritename) :
   ExposedObject<Rock, scripting::Rock>(this),
   physic(),
   on_ground(false),
-  last_movement(),
+  last_movement(0.0f, 0.0f),
   on_grab_script(),
   on_ungrab_script()
 {
@@ -74,7 +77,7 @@ void
 Rock::update(float dt_sec)
 {
   if (!is_grabbed())
-    m_col.m_movement = physic.get_movement(dt_sec);
+    m_col.set_movement(physic.get_movement(dt_sec));
 }
 
 void
@@ -98,6 +101,11 @@ Rock::collision_solid(const CollisionHit& hit)
     physic.set_velocity_x(0);
     on_ground = true;
   }
+
+  if (on_ground) {
+    // Full friction!
+    physic.set_velocity_x(physic.get_velocity_x() * (1.f - GROUND_FRICTION));
+  }
 }
 
 HitResponse
@@ -128,7 +136,7 @@ Rock::collision(GameObject& other, const CollisionHit& hit)
   if (!on_ground) {
     if (hit.bottom && physic.get_velocity_y() > 200) {
       auto moving_object = dynamic_cast<MovingObject*> (&other);
-      if (moving_object) {
+      if (moving_object && moving_object->get_group() != COLGROUP_TOUCHABLE) {
         //Getting a rock on the head hurts. A lot.
         moving_object->collision_tile(Tile::HURTS);
         physic.set_velocity_y(0);
@@ -144,8 +152,9 @@ void
 Rock::grab(MovingObject& object, const Vector& pos, Direction dir_)
 {
   Portable::grab(object, pos, dir_);
-  m_col.m_movement = pos - get_pos();
-  last_movement = m_col.m_movement;
+  Vector movement = pos - get_pos();
+  m_col.set_movement(movement);
+  last_movement = movement;
   set_group(COLGROUP_TOUCHABLE); //needed for lanterns catching willowisps
   on_ground = false;
 
@@ -157,19 +166,27 @@ Rock::grab(MovingObject& object, const Vector& pos, Direction dir_)
 void
 Rock::ungrab(MovingObject& object, Direction dir)
 {
+  auto player = dynamic_cast<Player*> (&object);
   set_group(COLGROUP_MOVING_STATIC);
   on_ground = false;
-  if (dir == Direction::UP) {
-    physic.set_velocity(0, -500);
-  } else if (dir == Direction::DOWN) {
-    physic.set_velocity(0, 500);
-  } else if (last_movement.norm() > 1) {
-    physic.set_velocity((dir == Direction::RIGHT) ? 200.0f : -200.0f, -200.0f);
-  } else {
-    physic.set_velocity(0, 0);
+  if (player)
+  {
+    if (player->is_swimming() || player->is_water_jumping())
+    {
+      float swimangle = player->get_swimming_angle();
+      physic.set_velocity(player->get_velocity() + Vector(std::cos(swimangle), std::sin(swimangle)));
+    }
+    else
+    {
+      physic.set_velocity_x(fabsf(player->get_physic().get_velocity_x()) < 1.f ? 0.f :
+        player->m_dir == Direction::LEFT ? -200.f : 200.f);
+      physic.set_velocity_y((dir == Direction::UP) ? -500.f : (dir == Direction::DOWN) ? 500.f :
+        (glm::length(last_movement) > 1) ? -200.f : 0.f);
+    }
   }
 
-  if (!on_ungrab_script.empty()) {
+  if (!on_ungrab_script.empty())
+  {
     Sector::get().run_script(on_ungrab_script, "Rock::on_ungrab");
   }
   Portable::ungrab(object, dir);
@@ -184,5 +201,18 @@ Rock::get_settings()
   return result;
 }
 
+void
+Rock::add_wind_velocity(const Vector& velocity, const Vector& end_speed)
+{
+  // only add velocity in the same direction as the wind
+  if (end_speed.x > 0 && physic.get_velocity_x() < end_speed.x)
+    physic.set_velocity_x(std::min(physic.get_velocity_x() + velocity.x, end_speed.x));
+  if (end_speed.x < 0 && physic.get_velocity_x() > end_speed.x)
+    physic.set_velocity_x(std::max(physic.get_velocity_x() + velocity.x, end_speed.x));
+  if (end_speed.y > 0 && physic.get_velocity_y() < end_speed.y)
+    physic.set_velocity_y(std::min(physic.get_velocity_y() + velocity.y, end_speed.y));
+  if (end_speed.y < 0 && physic.get_velocity_y() > end_speed.y)
+    physic.set_velocity_y(std::max(physic.get_velocity_y() + velocity.y, end_speed.y));
+}
 
 /* EOF */
