@@ -25,6 +25,7 @@
 Torch::Torch(const ReaderMapping& reader) :
   MovingObject(reader),
   ExposedObject<Torch, scripting::Torch>(this),
+  m_light_color(1.f, 1.f, 1.f),
   m_torch(),
   m_flame(SpriteManager::current()->create("images/objects/torch/flame.sprite")),
   m_flame_glow(SpriteManager::current()->create("images/objects/torch/flame_glow.sprite")),
@@ -40,11 +41,21 @@ Torch::Torch(const ReaderMapping& reader) :
   reader.get("burning", m_burning, true);
   reader.get("layer", m_layer, 0);
 
+  std::vector<float> vColor;
+  if (!reader.get("color", vColor)) vColor = { 1.f, 1.f, 1.f };
+
   m_torch = SpriteManager::current()->create(sprite_name);
   m_col.m_bbox.set_size(static_cast<float>(m_torch->get_width()),
                 static_cast<float>(m_torch->get_height()));
   m_flame_glow->set_blend(Blend::ADD);
   m_flame_light->set_blend(Blend::ADD);
+  if (vColor.size() >= 3)
+  {
+    m_light_color = Color(vColor);
+    m_flame->set_color(m_light_color);
+    m_flame_glow->set_color(m_light_color);
+    m_flame_light->set_color(m_light_color);
+  }
   set_group(COLGROUP_TOUCHABLE);
 }
 
@@ -54,16 +65,16 @@ Torch::draw(DrawingContext& context)
   if (m_burning)
   {
     m_flame->draw(context.color(), get_pos(), m_layer - 1);
+    m_flame->set_action(m_light_color.greyscale() >= 1.f ? "default" : "greyscale");
 
     m_flame_light->draw(context.light(), get_pos(), m_layer);
+    m_flame_light->set_action(m_light_color.greyscale() >= 1.f ? "default" : "greyscale");
+
+    m_flame_glow->draw(context.color(), get_pos(), m_layer - 1);
+    m_flame_glow->set_action(m_light_color.greyscale() >= 1.f ? "default" : "greyscale");
   }
 
   m_torch->draw(context.color(), get_pos(), m_layer - 1);
-
-  if (m_burning)
-  {
-    m_flame_glow->draw(context.color(), get_pos(), m_layer - 1);
-  }
 }
 
 void
@@ -90,8 +101,9 @@ Torch::get_settings()
   result.add_bool(_("Burning"), &m_burning, "burning", true);
   result.add_sprite(_("Sprite"), &sprite_name, "sprite", std::string("images/objects/torch/torch1.sprite"));
   result.add_int(_("Layer"), &m_layer, "layer", 0);
+  result.add_color(_("Color"), &m_light_color, "color", Color::WHITE);
 
-  result.reorder({"sprite", "layer", "x", "y"});
+  result.reorder({"sprite", "layer", "color", "x", "y"});
 
   return result;
 }
@@ -99,6 +111,9 @@ Torch::get_settings()
 void Torch::after_editor_set()
 {
   m_torch = SpriteManager::current()->create(sprite_name);
+  m_flame->set_color(m_light_color);
+  m_flame_glow->set_color(m_light_color);
+  m_flame_light->set_color(m_light_color);
 }
 
 bool
