@@ -5,19 +5,25 @@
 #include "scripting/badguy.hpp"
 #include "scripting/camera.hpp"
 #include "scripting/candle.hpp"
+#include "scripting/clouds.hpp"
+#include "scripting/custom_particles.hpp"
+#include "scripting/decal.hpp"
 #include "scripting/dispenser.hpp"
 #include "scripting/display_effect.hpp"
 #include "scripting/floating_image.hpp"
 #include "scripting/functions.hpp"
+#include "scripting/game_object_manager.hpp"
 #include "scripting/gradient.hpp"
 #include "scripting/level.hpp"
 #include "scripting/level_time.hpp"
 #include "scripting/particlesystem.hpp"
 #include "scripting/platform.hpp"
 #include "scripting/player.hpp"
+#include "scripting/rain.hpp"
 #include "scripting/rock.hpp"
 #include "scripting/scripted_object.hpp"
 #include "scripting/sector.hpp"
+#include "scripting/spotlight.hpp"
 #include "scripting/text.hpp"
 #include "scripting/text_array.hpp"
 #include "scripting/thunderstorm.hpp"
@@ -25,5 +31,6 @@
 #include "scripting/torch.hpp"
 #include "scripting/willowisp.hpp"
 #include "scripting/wind.hpp"
+#include "scripting/worldmap.hpp"
 
 /* EOF */
