@@ -29,7 +29,9 @@ class SquirrelError final : public std::exception
 {
 public:
   SquirrelError(HSQUIRRELVM v, const std::string& message) throw();
-  virtual ~SquirrelError() throw();
+  SquirrelError(const SquirrelError&) = default;
+  SquirrelError& operator=(const SquirrelError&) = default;
+  ~SquirrelError() throw() override;
 
   virtual const char* what() const throw() override;
 
