@@ -16,6 +16,8 @@
 
 #include "supertux/colorscheme.hpp"
 
+#include "editor/overlay_widget.hpp"
+#include "interface/control.hpp"
 #include "object/floating_text.hpp"
 #include "object/level_time.hpp"
 #include "object/text_object.hpp"
@@ -63,4 +65,8 @@ Color ColorScheme::Text::heading_color(1.f,1.f,0.6f);
 Color ColorScheme::Text::reference_color(0.2f,0.6f,1.f);
 Color ColorScheme::Text::normal_color(1.f,1.f,1.f);
 
+Color EditorOverlayWidget::text_autotile_available_color(1.f,1.f,0.6f);
+Color EditorOverlayWidget::text_autotile_active_color(1.f,1.f,1.f);
+Color EditorOverlayWidget::text_autotile_error_color(1.f,0.2f,0.1f);
+
 /* EOF */
