@@ -33,20 +33,11 @@ class Decal;
 /** Vertically or horizontally flip a level */
 class FlipLevelTransformer final : public LevelTransformer
 {
+public:
+  static void transform_flip(Flip& flip);
+  static void transform_path(float height, float obj_height, Path& path);
 public:
   virtual void transform_sector(Sector& sector) override;
-
-private:
-  Flip transform_flip(Flip flip);
-  void transform_path(float height, float obj_height, Path& path);
-  void transform_tilemap(float height, TileMap& tilemap);
-  void transform_moving_object(float height, MovingObject& object);
-  void transform_badguy(float height, BadGuy& badguy);
-  void transform_spawnpoint(float height, SpawnPoint& spawnpoint);
-  void transform_flower(Flower& flower);
-  void transform_platform(float height, Platform& platform);
-  void transform_block(float height, Block& block);
-  void transform_decal(float height, Decal& decal);
 };
 
 #endif
