@@ -16,9 +16,7 @@
 
 #include "supertux/globals.hpp"
 
-#include "supertux/gameconfig.hpp"
-
-std::unique_ptr<Config> g_config;
+Config* g_config;
 
 float g_game_time = 0;
 float g_real_time = 0;
