@@ -26,17 +26,21 @@
 #include "supertux/level_parser.hpp"
 #include "supertux/levelset.hpp"
 #include "supertux/menu/editor_levelset_menu.hpp"
+#include "supertux/menu/editor_delete_level_menu.hpp"
+#include "supertux/menu/editor_levelset_select_menu.hpp"
 #include "supertux/world.hpp"
 #include "util/file_system.hpp"
 
 EditorLevelSelectMenu::EditorLevelSelectMenu() :
-  m_levelset()
+  m_levelset(),
+  m_levelset_select_menu()
 {
   initialize();
 }
 
-EditorLevelSelectMenu::EditorLevelSelectMenu(std::unique_ptr<World> world) :
-  m_levelset()
+EditorLevelSelectMenu::EditorLevelSelectMenu(std::unique_ptr<World> world,EditorLevelsetSelectMenu* levelset_select_menu) :
+  m_levelset(),
+  m_levelset_select_menu(levelset_select_menu)
 {
   Editor::current()->set_world(std::move(world));
   initialize();
@@ -76,9 +80,9 @@ void EditorLevelSelectMenu::initialize() {
   if (PHYSFS_exists(worldmap_file.c_str())) {
     add_entry(-4, _("Edit Worldmap"));
   } else {
-    add_entry(-5, _("Create Worldmap"));
+    add_entry(-6, _("Create Worldmap"));
   }
-
+  add_entry(-5,_("Delete level"));
   add_hl();
   add_entry(-3, _("World Settings"));
   add_back(_("Back"),-2);
@@ -134,6 +138,14 @@ EditorLevelSelectMenu::create_item(bool worldmap)
   }
 }
 
+void
+EditorLevelSelectMenu::open_level(const std::string& filename)
+{
+  auto editor = Editor::current();
+  editor->set_level(filename);
+  MenuManager::instance().clear_menu_stack();
+}
+
 void
 EditorLevelSelectMenu::menu_action(MenuItem& item)
 {
@@ -141,9 +153,29 @@ EditorLevelSelectMenu::menu_action(MenuItem& item)
   World* world = editor->get_world();
   if (item.get_id() >= 0)
   {
-    editor->set_level(m_levelset->get_level_filename(item.get_id()));
 
-    MenuManager::instance().clear_menu_stack();
+    std::string file_name = m_levelset->get_level_filename(item.get_id());
+    std::string file_name_full = FileSystem::join(editor->get_level_directory(), file_name);
+
+    if (PHYSFS_exists((file_name_full + "~").c_str())) {
+      auto dialog = std::make_unique<Dialog>(/* passive = */ false, /* auto_clear_dialogs = */ false);
+      dialog->set_text(_("An auto-save recovery file was found. Would you like to restore the recovery\nfile and resume where you were before the editor crashed?"));
+      dialog->clear_buttons();
+      dialog->add_default_button(_("Yes"), [this, file_name] {
+        open_level(file_name + "~");
+        MenuManager::instance().set_dialog({});
+      });
+      dialog->add_button(_("No"), [this, file_name] {
+        Dialog::show_confirmation(_("This will delete the auto-save file. Are you sure?"), [this, file_name] {
+          open_level(file_name);
+        });
+      });
+      dialog->add_cancel_button(_("Cancel"));
+      MenuManager::instance().set_dialog(std::move(dialog));
+    } else {
+      open_level(file_name);
+    }
+
   } else {
     switch (item.get_id()) {
       case -1:
@@ -160,7 +192,15 @@ EditorLevelSelectMenu::menu_action(MenuItem& item)
         editor->set_level("worldmap.stwm");
         MenuManager::instance().clear_menu_stack();
         break;
-      case -5:
+      case -5: {
+        if (m_levelset->get_num_levels() > 0)
+        {
+          auto delete_menu = std::unique_ptr<Menu>(new EditorDeleteLevelMenu(m_levelset, this, m_levelset_select_menu));
+          MenuManager::instance().push_menu(std::move(delete_menu));
+        }
+        break;
+      }
+      case -6:
         create_worldmap();
         break;
       default:
@@ -168,5 +208,10 @@ EditorLevelSelectMenu::menu_action(MenuItem& item)
     }
   }
 }
-
+void
+EditorLevelSelectMenu::reload_menu()
+{
+  clear();
+  initialize();
+}
 /* EOF */
