@@ -21,19 +21,25 @@
 
 class Levelset;
 class World;
+class EditorLevelsetSelectMenu;
 
 class EditorLevelSelectMenu final : public Menu
 {
 private:
   std::unique_ptr<Levelset> m_levelset;
+  EditorLevelsetSelectMenu* m_levelset_select_menu;
 
 public:
   EditorLevelSelectMenu();
-  EditorLevelSelectMenu(std::unique_ptr<World> world);
-  ~EditorLevelSelectMenu();
+  EditorLevelSelectMenu(std::unique_ptr<World> world, EditorLevelsetSelectMenu* levelset_select_menu);
+  ~EditorLevelSelectMenu() override;
 
   void menu_action(MenuItem& item) override;
 
+  void open_level(const std::string& filename);
+
+  void reload_menu();
+
 private:
   void initialize();
   void create_level();
