@@ -23,7 +23,7 @@ class EditorObjectgroupMenu final : public Menu
 {
 public:
   EditorObjectgroupMenu();
-  ~EditorObjectgroupMenu();
+  ~EditorObjectgroupMenu() override;
 
   void menu_action(MenuItem& item) override;
 
