@@ -23,7 +23,7 @@ class EditorSectorsMenu final : public Menu
 {
 public:
   EditorSectorsMenu();
-  ~EditorSectorsMenu();
+  ~EditorSectorsMenu() override;
 
   void menu_action(MenuItem& item) override;
 
