@@ -63,7 +63,12 @@ public:
     EDITOR_SECTORS_MENU,
     EDITOR_SECTOR_MENU,
     EDITOR_LEVEL_MENU,
-    EDITOR_LEVELSET_MENU
+    EDITOR_LEVELSET_MENU,
+    PARTICLE_EDITOR_MENU,
+    PARTICLE_EDITOR_SAVE_AS,
+    PARTICLE_EDITOR_OPEN,
+    INTEGRATIONS_MENU,
+    ASSET_MENU
   };
 
 public:
