@@ -48,6 +48,16 @@ PlayerStatus::PlayerStatus() :
   }
 }
 
+void
+PlayerStatus::take_checkpoint_coins()
+{
+  int subtract_value = std::max(coins / 10, 25);
+  if (coins - subtract_value >= 0)
+    coins -= subtract_value;
+  else
+    coins = 0;
+}
+
 void PlayerStatus::reset()
 {
   coins = START_COINS;
@@ -63,8 +73,7 @@ PlayerStatus::get_max_coins() const
 bool
 PlayerStatus::can_reach_checkpoint() const
 {
-  return coins >= 25
-    && !GameSession::current()->get_reset_point_sectorname().empty();
+  return !GameSession::current()->get_reset_point_sectorname().empty();
 }
 
 void
