@@ -19,7 +19,11 @@
 #define HEADER_SUPERTUX_SUPERTUX_SCREEN_MANAGER_HPP
 
 #include <memory>
+#include <SDL2/SDL.h>
 
+#include "config.h"
+
+#include "control/mobile_controller.hpp"
 #include "squirrel/squirrel_thread_queue.hpp"
 #include "supertux/screen.hpp"
 #include "util/currenton.hpp"
@@ -40,7 +44,7 @@ class ScreenManager final : public Currenton<ScreenManager>
 {
 public:
   ScreenManager(VideoSystem& video_system, InputManager& input_manager);
-  ~ScreenManager();
+  ~ScreenManager() override;
 
   void run();
   void quit(std::unique_ptr<ScreenFade> fade = {});
@@ -53,6 +57,8 @@ public:
   void pop_screen(std::unique_ptr<ScreenFade> fade = {});
   void set_screen_fade(std::unique_ptr<ScreenFade> fade);
 
+  void loop_iter();
+
 private:
   struct FPS_Stats;
   void draw_fps(DrawingContext& context, FPS_Stats& fps_statistics);
@@ -68,6 +74,15 @@ private:
   std::unique_ptr<MenuStorage> m_menu_storage;
   std::unique_ptr<MenuManager> m_menu_manager;
   std::unique_ptr<ControllerHUD> m_controller_hud;
+#ifdef ENABLE_TOUCHSCREEN_SUPPORT
+  MobileController m_mobile_controller;
+#endif
+
+  Uint32 last_ticks;
+  Uint32 elapsed_ticks;
+  const Uint32 ms_per_step;
+  const float seconds_per_step;
+  std::unique_ptr<FPS_Stats> m_fps_statistics;
 
   float m_speed;
   struct Action
